#!/usr/bin/perl

use strict;
use warnings;
use JSON;
use Image::Magick;
local $ENV{PATH} = "/sbin:/bin:/usr/sbin:/usr/bin:/usr/local/sbin:/usr/local/bin";

    my $dir = 'upload/';
    my $conf = {
           mp4         => $dir . 'demo.mp4',
           preview     => $dir . 'preview.jpg',
           json        => $dir . 'video.json',
           capture     => $dir . 'capture',
           log         => $dir . 'capture.log',
           capture_sec => 5
       };

    my $progress = {
           cur       => 0,
           frame     => 0,
           drop      => 0,
           nb_frames => 0,
           err       => 0,
           dir       => $dir,
           file      => 0,
           thumbnail => 0
       };

    if (! -d $dir) {
        $progress->{err} = 1;
        $progress->{msg} = 'NotFound: $dir';
    }

    if ($progress->{err}) {
        &print_json($progress);
        exit;
    }

    &get_progress();

    if ($progress->{err}) {

        &print_json($progress);

    } elsif ($progress->{cur} == 100) {

        &make_preview_image();

    }

    &print_json($progress);

sub print_json {

    my $data = shift;

    print "Content-type: application/json; charset=utf-8\n";
    print "X-Content-Type-Options: nosniff\n\n";
    print to_json($data);
    exit;

}

sub get_progress {

    open(IN, "< $conf->{json}");
    my @read_data = <IN>;
    close(IN);
    chomp(@read_data);

    my $json = from_json($read_data[0]);

    foreach (@{$json->{streams}}) {

        if ($_->{codec_type} =~ /video/) {

            $progress->{nb_frames} = $_->{nb_frames};
            last;

        }

    }

    sleep(3);

    open(IN, "< $conf->{log}");
    my @in = <IN>;
    close(IN);
    chomp(@in);

    my @tmp = ();

    foreach (@in) {

        if ($_ =~ /(error|fail|Invalid|No such file or directory)/) {

            ## skip error
            if ($_ =~ /decode_slice_header error/) { next; }

            $progress->{err} = 1;
            $progress->{msg} = "Error: check $conf->{log}";
            return;
        }

        if ($_ =~ /frame/) {
            $_ =~ s/\r/::/g;
            push(@tmp, split('::', $_));
        }
    }

    my $cur = pop(@tmp);
    $cur =~ /(frame=(\s+)?(\d+)).+(drop=(\s+)?(\d+))/;

    if ($3) { $progress->{frame} = $3; }
    if ($6) { $progress->{drop} = $6; }

    $progress->{cur} = int(($progress->{frame} + $progress->{drop}) / $progress->{nb_frames} * 100);

    return;

}

sub make_preview_image {

    my $i = 1;
    my $n = 1;

    my $allitem = '';
    my $img = Image::Magick->new;

    my @files = glob("$conf->{capture}/*.jpg");
    chomp(@files);

    foreach (@files) {

        if ($i == $conf->{capture_sec}) {

            my $rename = $n . '.jpg';
            `mv $_ "$conf->{capture}/$rename"`;

            $img->Read("$conf->{capture}/$rename");
            $img->Resize(geometry => '160x90');
            $img->Set(Gravity => 'Center');
            $img->Crop(geometry => '120x90+0+0');
            $img->Set(page => '120x90+0+0');

            $allitem = $img->Append(stack => 'false');

            $progress->{file} = $n;

            $i = 1;
            $n++;

        } else {
            `rm $_`;
        }

        $i++;

    }

    $allitem->Write($conf->{preview});

    undef($img);
    undef($allitem);

    my $end = $progress->{file};
    if ($end > 20) { $end = 20; }
    my $rand = &rand_num_range(1, $end);

    `cp "$conf->{capture}/$rand.jpg" "$dir/thumbnail.jpg"`;
    $progress->{thumbnail} = "$dir/thumbnail.jpg";

    return;

}

sub rand_num_range {

    my ($start, $end) = @_;
    if ($start > $end) { return(0); }

    my @array = ();

    for (my $i = $start; $i <= $end; $i++) {
        push(@array, $i);
    }

    my $rand = int(rand(scalar(@array)));
    return($array[$rand]);

}
